import {DependencyList, useMemo} from 'react';
import {GraphqlQueryStore, IGraphqlQueryStoreParams} from '@proscom/prostore-apollo/lib/GraphqlQueryStore';
import { useRequestStore } from '@proscom/prostore-react/lib/useRequestStore';
import { useContextApolloClient } from './ApolloClientsContext';
import {ApolloClient} from 'apollo-client';

export interface IUseGraphqlQueryParams<Vars, Data> {
  client?: string | ApolloClient<any>;
  queryOptions?: IGraphqlQueryStoreParams<Vars, Data>;
  variables?: Vars;
}

export function useGraphqlQuery<Vars, Data>({ client, queryOptions, variables }: IUseGraphqlQueryParams<Vars, Data>) {
  const actualClient = useContextApolloClient(client);

  const store = useMemo(
    () =>
      new GraphqlQueryStore({
        client: actualClient,
        ...queryOptions,
      }),
    [actualClient, queryOptions],
  );

  return useRequestStore(store, variables);
}

export interface IUseGraphQueryVarsParams<Vars, Data> extends IUseGraphqlQueryParams<Vars, Data> {
  variableCreator: [
    () => Vars,
    DependencyList
  ];
}

export function useGraphqlQueryVars<Vars, Data>({ variableCreator, ...queryProps }: IUseGraphQueryVarsParams<Vars, Data>) {
  const variables = useMemo(...variableCreator);
  return useGraphqlQuery({ ...queryProps, variables });
}
